@@ -56,14 +56,14 @@ AdaBagModel <- function(
     gridinfo = new_gridinfo(
       param = c("mfinal", "maxdepth"),
       get_values = c(
-        function(n, ...) round(seq_range(0, 25, c(1, 200), n + 1)),
+        function(n, ...) round_int(seq_range(0, 25, c(1, 200), n + 1)),
         function(n, ...) seq_len(min(n, 30))
       )
     ),
 
     fit = function(formula, data, weights, mfinal, ...) {
-      adabag::bagging(formula, data = as.data.frame(data), mfinal = mfinal,
-                      control = list(...))
+      adabag::bagging(formula, data = as.data.frame(formula, data),
+                      mfinal = mfinal, control = list(...))
     },
 
     predict = function(object, newdata, ...) {
