@@ -75,7 +75,7 @@ FDAModel <- function(
         function(n, data, ...) {
           model_fit <- fit(data, model = EarthModel(pmethod = "none"))
           max_terms <- min(2 + 0.75 * nrow(model_fit$dirs), 200)
-          round(seq(2, max_terms, length = n))
+          round_int(seq(2, max_terms, length = n))
         },
         function(n, ...) head(1:2, n)
       ),
@@ -83,7 +83,8 @@ FDAModel <- function(
     ),
 
     fit = function(formula, data, weights, ...) {
-      mda::fda(formula, data = as.data.frame(data), weights = weights, ...)
+      mda::fda(formula, data = as.data.frame(formula, data), weights = weights,
+               ...)
     },
 
     predict = function(object, newdata, prior = object$prior, ...) {
