@@ -63,7 +63,7 @@ LDAModel <- function(
     ),
 
     fit = function(formula, data, weights, dimen, use, ...) {
-      model_fit <- MASS::lda(formula, data = as.data.frame(data), ...)
+      model_fit <- MASS::lda(formula, data = as.data.frame(formula, data), ...)
       model_fit$dimen <- if (missing(dimen)) length(model_fit$svd) else dimen
       model_fit$use <- use
       model_fit
