@@ -59,7 +59,7 @@ NNetModel <- function(
     gridinfo = new_gridinfo(
       param = c("size", "decay"),
       get_values = c(
-        function(n, ...) round(seq_range(1, 2, c(1, 20), length = n)),
+        function(n, ...) round_int(seq_range(1, 2, c(1, 20), n)),
         function(n, ...) c(0, 10^seq_inner(-5, 1, n - 1))
       )
     ),
@@ -83,8 +83,8 @@ NNetModel <- function(
         model_fit$xlevels <- list()
         model_fit
       } else {
-        nnet::nnet(formula, data = as.data.frame(data), weights = weights,
-                   linout = linout, ...)
+        nnet::nnet(formula, data = as.data.frame(formula, data),
+                   weights = weights, linout = linout, ...)
       }
     },
 
