@@ -21,9 +21,9 @@
 #'   these with which to calculate performance.  If not specified, default
 #'   metrics defined in the \link{performance} functions are used.  Recipe
 #'   selection is based on the first calculated metric.
+#' @param cutoff argument passed to the \code{metrics} functions.
 #' @param stat function or character string naming a function to compute a
 #'   summary statistic on resampled metric values for recipe selection.
-#' @param cutoff argument passed to the \code{metrics} functions.
 #'
 #' @return \code{SelectedModelFrame} or \code{SelectedModelRecipe} class object
 #' that inherits from \code{SelectedInput} and \code{ModelFrame} or
@@ -60,8 +60,8 @@ SelectedInput <- function(...) {
 #'
 SelectedInput.formula <- function(
   ..., data, control = MachineShop::settings("control"), metrics = NULL,
-  stat = MachineShop::settings("stat.TrainingParams"),
-  cutoff = MachineShop::settings("cutoff")
+  cutoff = MachineShop::settings("cutoff"),
+  stat = MachineShop::settings("stat.TrainingParams")
 ) {
   inputs <- list(...)
   if (!all(map("logi", is, inputs, "formula"))) {
@@ -79,8 +79,8 @@ SelectedInput.formula <- function(
 #'
 SelectedInput.matrix <- function(
   ..., y, control = MachineShop::settings("control"), metrics = NULL,
-  stat = MachineShop::settings("stat.TrainingParams"),
-  cutoff = MachineShop::settings("cutoff")
+  cutoff = MachineShop::settings("cutoff"),
+  stat = MachineShop::settings("stat.TrainingParams")
 ) {
   inputs <- list(...)
   if (!all(map("logi", is, inputs, "matrix"))) {
@@ -96,8 +96,8 @@ SelectedInput.matrix <- function(
 #'
 SelectedInput.ModelFrame <- function(
   ..., control = MachineShop::settings("control"), metrics = NULL,
-  stat = MachineShop::settings("stat.TrainingParams"),
-  cutoff = MachineShop::settings("cutoff")
+  cutoff = MachineShop::settings("cutoff"),
+  stat = MachineShop::settings("stat.TrainingParams")
 ) {
 
   inputs <- list(...)
@@ -115,7 +115,7 @@ SelectedInput.ModelFrame <- function(
     throw(Error("ModelFrames have different response variables."))
   }
 
-  default_names <- map("char", training_names.MLInput, inputs)
+  default_names <- map("char", class, inputs)
   names(inputs) <- make_names_along(inputs, default_names)
   data <- NULL
   for (i in seq_along(inputs)) {
@@ -125,10 +125,10 @@ SelectedInput.ModelFrame <- function(
   new("SelectedModelFrame", ModelFrame(data),
     inputs = ListOf(map(terms, inputs)),
     params = TrainingParams(
-      control = as.MLControl(control),
+      control = control,
       metrics = metrics,
-      stat = stat,
-      cutoff = cutoff
+      cutoff = cutoff,
+      stat = stat
     )
   )
 
@@ -139,8 +139,8 @@ SelectedInput.ModelFrame <- function(
 #'
 SelectedInput.recipe <- function(
   ..., control = MachineShop::settings("control"), metrics = NULL,
-  stat = MachineShop::settings("stat.TrainingParams"),
-  cutoff = MachineShop::settings("cutoff")
+  cutoff = MachineShop::settings("cutoff"),
+  stat = MachineShop::settings("stat.TrainingParams")
 ) {
 
   inputs <- list(...)
@@ -163,12 +163,12 @@ SelectedInput.recipe <- function(
     throw(Error("Recipes have different non-predictor variables."))
   }
 
-  default_names <- map("char", training_names.MLInput, inputs)
+  default_names <- map("char", class, inputs)
   names(inputs) <- make_names_along(inputs, default_names)
   data <- NULL
   for (i in seq_along(inputs)) {
     data <- combine_data_frames(as.data.frame(inputs[[i]]), data)
-    inputs[[i]] <- recipe(inputs[[i]], tibble(.rows = nrow(data)))
+    inputs[[i]] <- update(inputs[[i]], data = tibble(.rows = nrow(data)))
   }
 
   outcome_vars <- get_info(inputs[[1]], roles = "outcome")$variable
@@ -185,10 +185,10 @@ SelectedInput.recipe <- function(
   new("SelectedModelRecipe", new("ModelRecipe", rec),
     inputs = ListOf(inputs),
     params = TrainingParams(
-      control = as.MLControl(control),
+      control = control,
       metrics = metrics,
-      stat = stat,
-      cutoff = cutoff
+      cutoff = cutoff,
+      stat = stat
     )
   )
 
@@ -203,24 +203,18 @@ SelectedInput.list <- function(x, ...) {
 
 
 .fit.SelectedInput <- function(object, ...) {
-  grid <- tibble(id = map("char", slot, object@inputs, "id"))
-  step <- resample_selection(
-    object, ..., grid = grid, params = object@params, id = object@id,
-    name = "SelectedInput"
-  )
-  input <- update(object, grid[step@grid$selected, ])
-  push(step, fit(input, ...))
+  fit_optim(object, ...)
 }
 
 
-update.SelectedInput <- function(object, params = list(), new_id = FALSE, ...) {
+update.SelectedInput <- function(object, params = list(), ...) {
   object <- as(object, "SelectedInput")
   new_params <- as(object, "list")
   new_params[names(params)] <- params
   objects <- new_params$objects
   new_params[c("objects", "id")] <- NULL
   res <- do.call(SelectedInput, c(objects, new_params))
-  if (!new_id) res@id <- object@id
+  res@id <- object@id
   res
 }
 
@@ -242,7 +236,7 @@ update.SelectedModelRecipe <- function(object, params = list(), ...) {
   object <- subset_selected(object, "inputs", params$id)
   data <- as.data.frame(object)
   object@inputs <- ListOf(map(function(x) {
-    rec <- ModelRecipe(recipe(x, data[unique(summary(x)$variable)]))
+    rec <- ModelRecipe(update(x, data = data[unique(summary(x)$variable)]))
     rec@id <- x@id
     rec
   }, object@inputs))
@@ -267,15 +261,15 @@ update.SelectedModelRecipe <- function(object, params = list(), ...) {
 #'   these with which to calculate performance.  If not specified, default
 #'   metrics defined in the \link{performance} functions are used.  Recipe
 #'   selection is based on the first calculated metric.
+#' @param cutoff argument passed to the \code{metrics} functions.
 #' @param stat function or character string naming a function to compute a
 #'   summary statistic on resampled metric values for recipe tuning.
-#' @param cutoff argument passed to the \code{metrics} functions.
 #' @param ... arguments passed to other methods.
 #'
 #' @return \code{TunedModelRecipe} class object that inherits from
 #' \code{TunedInput} and \code{recipe}.
 #'
-#' @seealso \code{\link{fit}}, \code{\link{resample}}
+#' @seealso \code{\link{fit}}, \code{\link{resample}}, \code{\link{set_optim}}
 #'
 #' @examples
 #' library(recipes)
@@ -299,17 +293,17 @@ TunedInput <- function(object, ...) {
 #'
 TunedInput.recipe <- function(
   object, grid = expand_steps(), control = MachineShop::settings("control"),
-  metrics = NULL, stat = MachineShop::settings("stat.TrainingParams"),
-  cutoff = MachineShop::settings("cutoff"), ...
+  metrics = NULL, cutoff = MachineShop::settings("cutoff"),
+  stat = MachineShop::settings("stat.TrainingParams"), ...
 ) {
 
   object <- new("TunedModelRecipe", ModelRecipe(object),
     grid = grid,
     params = TrainingParams(
-      control = as.MLControl(control),
+      control = control,
       metrics = metrics,
-      stat = stat,
-      cutoff = cutoff
+      cutoff = cutoff,
+      stat = stat
     )
   )
 
@@ -331,16 +325,16 @@ TunedInput.recipe <- function(
 
 
 .fit.TunedModelRecipe <- function(object, ...) {
-  grid <- object@grid
-  object_input <- as(object, "ModelRecipe")
-  if (prod(size(grid)) > 0) {
-    step <- resample_selection(
-      object_input, ..., grid = grid, params = object@params, id = object@id,
-      name = "TunedInput"
-    )
-    input <- update(object_input, grid[step@grid$selected, ])
-    push(step, fit(input, ...))
+  fit_optim(object, ...)
+}
+
+
+update.TunedModelRecipe <- function(
+  object, params = NULL, ...
+) {
+  if (is.list(params)) {
+    update(as(object, "ModelRecipe"), params = params, new_id = object@id)
   } else {
-    fit(object_input, ...)
+    object
   }
 }
