@@ -42,7 +42,7 @@ expand_model <- function(object, ..., random = FALSE) {
 
 .expand_model.tbl_df <- function(object, model, ...) {
   models <- map(function(params) {
-    do.call(update, list(model, params, new_id = TRUE), quote = TRUE)
+    do.call(update, list(model, params = params, new_id = TRUE), quote = TRUE)
   }, split(object, seq_len(max(nrow(object), 1))))
   names(models) <- paste0(models[[1]]@name, ".", names(models))
   models
@@ -62,17 +62,19 @@ expand_model <- function(object, ..., random = FALSE) {
 #' @rdname expand_modelgrid-methods
 #'
 #' @param ... arguments passed from the generic function to its methods and from
-#'   the \code{TunedModel} method to others.  The first arguments of
-#'   \code{expand_modelgrid} methods are positional and, as such, must be
-#'   given first in calls to them.
+#'   the \code{MLModel} and \code{MLModelFunction} methods to others.  The
+#'   first argument of each \code{expand_modelgrid} method is positional and, as
+#'   such, must be given first in calls to them.
 #' @param formula,data \link[=formula]{formula} defining the model predictor and
 #'   response variables and a \link[=data.frame]{data frame} containing them.
 #' @param x,y \link{matrix} and object containing predictor and response
 #'   variables.
 #' @param input \link[=inputs]{input} object defining and containing the model
 #'   predictor and response variables.
-#' @param model \code{\link{TunedModel}} object.  Can be given first followed by
-#'   any of the variable specifications.
+#' @param object model \link[=ModelSpecification]{specification}.
+#' @param model \link[=models]{model} function, function name, or object; or
+#'   another object that can be \link[=as.MLModel]{coerced} to a model.  A model
+#'   can be given first followed by any of the variable specifications.
 #' @param info logical indicating whether to return model-defined grid
 #'   construction information rather than the grid values.
 #'
@@ -121,44 +123,109 @@ expand_modelgrid.formula <- function(formula, data, model, info = FALSE, ...) {
 #' @rdname expand_modelgrid-methods
 #'
 expand_modelgrid.matrix <- function(x, y, model, info = FALSE, ...) {
-  expand_modelgrid(model, x, y, info = info)
+  .expand_modelgrid(as.MLModel(model), x, y, info = info)
 }
 
 
 #' @rdname expand_modelgrid-methods
 #'
 expand_modelgrid.ModelFrame <- function(input, model, info = FALSE, ...) {
-  expand_modelgrid(model, input, info = info)
+  .expand_modelgrid(as.MLModel(model), input, info = info)
 }
 
 
 #' @rdname expand_modelgrid-methods
 #'
 expand_modelgrid.recipe <- function(input, model, info = FALSE, ...) {
-  expand_modelgrid(model, input, info = info)
+  .expand_modelgrid(as.MLModel(model), input, info = info)
 }
 
 
 #' @rdname expand_modelgrid-methods
 #'
-expand_modelgrid.TunedModel <- function(model, ..., info = FALSE) {
-  if (info) {
-    model@gridinfo
-  } else {
-    .expand_modelgrid(model@grid, ..., model = model@model)
+expand_modelgrid.ModelSpecification <- function(object, ...) {
+  grid <- object@grid
+  if (is_empty(grid)) {
+    map_slots(function(object) {
+      types <- c("ModeledInput", "SelectedInput", "SelectedModel")
+      found <- map("logi", is, list(object), types)
+      if (any(found)) {
+        throw(Error(
+          "Cannot expand the tuning grids of a ModelSpecification containing ",
+          types[found], "."
+        ))
+      }
+    }, object)
+
+    get_modelgrids <- function(input) {
+      map_slots(function(model) {
+        get_grid(model, input)$params
+      }, object, names = c("model", "models"))
+    }
+
+    make_grid <- function(grids) {
+      expand_params(grids[!map("logi", is_empty, grids)])
+    }
+
+    input_grid <- make_grid(map_slots(function(input) {
+      get_grid(input)$params
+    }, object, names = c("input", "inputs")))
+
+    if (nrow(input_grid)) {
+      grid <- NULL
+      for (i in seq_len(nrow(input_grid))) {
+        params <- input_grid[i, ]
+        input <- update_slots(object@input, params = params)
+        grid <- rbind(grid, make_grid(c(params, get_modelgrids(input))))
+      }
+    } else {
+      grid <- make_grid(get_modelgrids(object@input))
+    }
   }
+  grid
 }
 
 
-.expand_modelgrid <- function(grid, ...) {
+#' @rdname expand_modelgrid-methods
+#'
+expand_modelgrid.MLModel <- function(model, ...) {
+  .expand_modelgrid(model, ...)
+}
+
+
+#' @rdname expand_modelgrid-methods
+#'
+expand_modelgrid.MLModelFunction <- function(model, ...) {
+  expand_modelgrid(as.MLModel(model), ...)
+}
+
+
+.expand_modelgrid <- function(object, ...) {
   UseMethod(".expand_modelgrid")
 }
 
 
-.expand_modelgrid.TuningGrid <- function(grid, input, ..., model) {
+.expand_modelgrid.MLModel <- function(object, ..., info = FALSE) {
+  if (info) object@gridinfo else tibble()
+}
+
+
+.expand_modelgrid.TunedModel <- function(object, ..., info = FALSE) {
+  if (info) {
+    NextMethod()
+  } else {
+    random <- if (is_optim_method(object, "RandomGridSearch")) {
+      get_optim_field(object, "size")
+    } else FALSE
+    .expand_modelgrid(object@grid, ..., model = object@model, random = random)
+  }
+}
+
+
+.expand_modelgrid.TuningGrid <- function(object, input, ..., model, random) {
   gridinfo <- model@gridinfo
-  size <- grid@size
-  random <- grid@random
+  size <- object@size
+  if (object@random) random <- object@random
   mf <- NULL
 
   not_dup <- function(x) !duplicated(x, fromLast = TRUE)
@@ -205,13 +272,12 @@ expand_modelgrid.TunedModel <- function(model, ..., info = FALSE) {
 }
 
 
-.expand_modelgrid.ParameterGrid <- function(grid, input, ..., model) {
-  grid <- if (nrow(grid)) {
-    needs_data <- any(dials::has_unknowns(grid$object))
+.expand_modelgrid.ParameterGrid <- function(object, input, ..., model, random) {
+  grid <- if (nrow(object)) {
+    needs_data <- any(dials::has_unknowns(object$object))
     if (needs_data) {
       if (missing(input)) return(NULL)
       mf <- ModelFrame(input, ..., na.rm = FALSE)
-      model <- as.MLModel(model)
       data <- switch(model@predictor_encoding,
         "model.frame" = {
           mf_terms <- attributes(terms(mf))
@@ -221,10 +287,10 @@ expand_modelgrid.TunedModel <- function(model, ..., info = FALSE) {
         },
         "model.matrix" = model.matrix(mf, intercept = FALSE)
       )
-      grid <- dials::finalize(grid, x = data)
+      object <- dials::finalize(object, x = data)
     }
-    params <- map(dials::value_seq, grid$object, grid@size)
-    expand_params(params, random = grid@random)
+    params <- map(dials::value_seq, object$object, object@size)
+    expand_params(params, random = if (object@random) object@random else random)
   } else {
     tibble()
   }
@@ -233,9 +299,8 @@ expand_modelgrid.TunedModel <- function(model, ..., info = FALSE) {
 }
 
 
-.expand_modelgrid.tbl_df <- function(grid, ...) {
-  if (!nrow(grid)) grid <- tibble(.rows = 1)
-  if (ncol(grid)) grid[!duplicated(grid), ] else grid
+.expand_modelgrid.tbl_df <- function(object, ...) {
+  if (ncol(object)) object[!duplicated(object), ] else object
 }
 
 
@@ -243,7 +308,7 @@ expand_modelgrid.TunedModel <- function(model, ..., info = FALSE) {
 #'
 #' Create a grid of parameter values from all combinations of supplied inputs.
 #'
-#' @param ... named vectors or factors or a list of these containing the
+#' @param ... named data frames or vectors or a list of these containing the
 #'   parameter values over which to create the grid.
 #' @param random number of points to be randomly sampled from the parameter grid
 #'   or \code{FALSE} if all points are to be returned.
@@ -268,13 +333,12 @@ expand_modelgrid.TunedModel <- function(model, ..., info = FALSE) {
 #' }
 #'
 expand_params <- function(..., random = FALSE) {
+  x <- list(...)
+  if (is_one_element(x, "list") && is.null(names(x))) x <- x[[1]]
   if (random) {
-    x <- list(...)
-    if (is_one_element(x, "list") && is.null(names(x))) x <- x[[1]]
-    sample_params(x, size = random)
+    random_grid(x, size = random)
   } else {
-    as_tibble(expand.grid(..., KEEP.OUT.ATTRS = FALSE,
-                          stringsAsFactors = FALSE))
+    regular_grid(x)
   }
 }
 
@@ -339,18 +403,7 @@ expand_steps <- function(..., random = FALSE) {
     throw(Error("Step names must be unique."))
   }
 
-  grid <- expand_params(unlist(steps, recursive = FALSE), random = random)
-  recipe_grid <- tibble(.rows = nrow(grid))
-
-  start <- 1
-  for (name in step_names) {
-    n <- length(steps[[name]])
-    x <- grid[seq(start, length = n)]
-    names(x) <- substring(names(x), nchar(name) + 2)
-    recipe_grid[[name]] <- x
-    start <- start + n
-  }
-
-  RecipeGrid(recipe_grid)
+  steps <- map(expand_params, steps, random = random)
+  RecipeGrid(expand_params(steps, random = random))
 
 }
