@@ -8,6 +8,9 @@
 #' @param recipe existing \link[recipes]{recipe} object.
 #' @param x,size number of counts and trials for the specification of a
 #'   \code{\link{BinomialVariate}} outcome.
+#' @param group variable defining groupings of cases to keep together when
+#'   folds are constructed for \link[=controls]{cross-validation}
+#'   [default: none].
 #' @param stratum variable to use in conducting stratified \link{resample}
 #'   estimation of model performance.
 #' @param weight numeric variable of case weights for model
@@ -50,19 +53,29 @@ role_binom <- function(recipe, x, size) {
     recipes::add_role(recipe, x, new_role = "binom_x") %>%
       recipes::add_role(size, new_role = "binom_size")
   } else {
-    throw(Error("binomial 'x' and 'size' variables must be specified"))
+    throw(Error("Binomial 'x' and 'size' variables must be specified."))
   }
 }
 
 
 #' @rdname recipe_roles
 #'
-role_case <- function(recipe, stratum, weight, replace = FALSE) {
-  stratum <- as.character(substitute(stratum))
-  weight <- as.character(substitute(weight))
+role_case <- function(recipe, group, stratum, weight, replace = FALSE) {
+  comp_names <- map(
+    as.character,
+    eval(substitute(alist(
+      group = group,
+      stratum = stratum,
+      weight = weight
+    )))
+  )
   f <- if (replace) recipes::update_role else recipes::add_role
-  if (nzchar(stratum)) recipe <- f(recipe, stratum, new_role = "case_stratum")
-  if (nzchar(weight)) recipe <- f(recipe, weight, new_role = "case_weight")
+  for (type in names(comp_names)) {
+    name <- comp_names[[type]]
+    if (nzchar(name)) {
+      recipe <- f(recipe, name, new_role = paste0("case_", type))
+    }
+  }
   recipe
 }
 
@@ -88,6 +101,6 @@ role_surv <- function(recipe, time, event) {
     }
     recipe
   } else {
-    throw(Error("a survival 'time' variable must be specified"))
+    throw(Error("A survival 'time' variable must be specified."))
   }
 }
