@@ -71,11 +71,14 @@ VariableImportance.numeric <- function(object, ...) {
 #'       change to compute in comparing model predictive performances between
 #'       datasets with and without permuted values.  The choices are difference
 #'       (\code{"-"}) and ratio (\code{"/"}).}
-#'     \item{\code{stats = c(mean = "base::mean")}}{function, function name, or
-#'       vector of these with which to compute summary statistics on the set of
-#'       variable importance values from the permuted datasets.}
+#'     \item{\code{stats = MachineShop::settings("stat.TrainingParams")}}{
+#'       function, function name, or vector of these with which to compute
+#'       summary statistics on the set of variable importance values from the
+#'       permuted datasets.}
 #'     \item{\code{na.rm = TRUE}}{logical indicating whether to exclude missing
 #'       variable importance values from the calculation of summary statistics.}
+#'     \item{\code{progress = TRUE}}{logical indicating whether to display
+#'       iterative progress during computation.}
 #'   }
 #'
 #' @return \code{VariableImportance} class object.
@@ -114,8 +117,7 @@ varimp <- function(object, method = c("permute", "model"), scale = TRUE, ...) {
 
   switch(match.arg(method),
     "model" = {
-      args <- dep_varimpargs(...)
-      vi <- do.call(model@varimp, c(list(unMLModelFit(object)), args))
+      vi <- model@varimp(unMLModelFit(object), ...)
       if (is.null(vi)) vi <- varimp_permute(object)
     },
     "permute" = {
@@ -130,7 +132,8 @@ varimp <- function(object, method = c("permute", "model"), scale = TRUE, ...) {
 varimp_permute <- function(
   object, select = NULL, samples = 1, prop = numeric(), size = integer(),
   times = numeric(), metric = NULL, compare = c("-", "/"),
-  stats = c(mean = "base::mean"), na.rm = TRUE
+  stats = MachineShop::settings("stat.TrainingParams"), na.rm = TRUE,
+  progress = TRUE
 ) {
   input <- as.MLModel(object)@input
   data <- if (is.data.frame(input)) input else as.data.frame(input)
@@ -170,19 +173,24 @@ varimp_permute <- function(
     map(function(x) stats(if (na.rm) na.omit(x) else x), as.data.frame(x))
   }
 
+  progress <- if (throw(check_logical(progress))) {
+    pb <- progress_bar$new(
+      format = "varimp permute [:bar] :percent | :eta",
+      total = samples * length(pred_names),
+      show_after = 1
+    )
+    on.exit(pb$terminate())
+    function() pb$tick()
+  } else {
+    function() NULL
+  }
+
   num_workers <- getDoParWorkers()
   work <- pred_names
   length(work) <- num_workers * ceiling(length(work) / num_workers)
   work_pred_names <- map(na.omit, split(work, seq_len(num_workers)))
   seeds <- rand_int(samples)
 
-  pb <- progress_bar$new(
-    format = "varimp permute [:bar] :percent | :eta",
-    total = samples * length(pred_names),
-    show_after = 1
-  )
-  on.exit(pb$terminate())
-
   foreach(
     pred_names = work_pred_names[lengths(work_pred_names) > 0],
     .combine = rbind,
@@ -199,7 +207,7 @@ varimp_permute <- function(
       base_perf[s] <- if (s == 1 || subset) {
         newdata <- if (subset) data[inds$i, ] else data
         obs <- response(object, newdata)
-        pred <- predict(object, newdata, times = times, type = "prob")
+        pred <- predict(object, newdata, times = times, type = "default")
         if (is.null(metric)) {
           metric <- as.MLMetric(get_perf_metrics(obs, pred)[[1]])
         }
@@ -210,10 +218,10 @@ varimp_permute <- function(
       for (name in pred_names) {
         x <- newdata[[name]]
         newdata[[name]] <- data[[name]][inds$j]
-        pred <- predict(object, newdata, times = times, type = "prob")
+        pred <- predict(object, newdata, times = times, type = "default")
         perf[s, name] <- metric(obs, pred)[1]
         newdata[[name]] <- x
-        pb$tick()
+        progress()
       }
     }
     sims <- varimp(perf, base_perf, metric@maximize)
